/*
* This file is part of Firestorm NIDS
* Copyright (c) 2001 Gianni Tedesco
*
*    Firestorm is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    Firestorm is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with Firestorm; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <match_tcp.h>
#include <s_util.h>

PLUGIN_DECLARE_STD_IMPORTS;

proc_matcher_add	add_matcher;

#define CHECK_TCP(x) \
	(( TRANSPORT_LAYER(x).id == IPPROTO_TCP ) ? 1 : 0)

void *flags_validate(char *token, p_kv opts)
{
	p_tcpflag_priv priv;
	char *tmp;
	
	if ( !(priv=my_malloc(sizeof(tcpflag_priv_t))) ) return NULL;
	memset(priv, 0, sizeof(tcpflag_priv_t));
	
	for (tmp=token; *tmp != '\0'; tmp++ ) {
		switch(*tmp) {
			case 'f':
			case 'F':
				priv->tcp_flags |= TCP_FIN;
				break;
				
			case 's':
			case 'S':
				priv->tcp_flags |= TCP_SYN;
				break;
				
			case 'r':
			case 'R':
				priv->tcp_flags |= TCP_RST;
				break;

			case 'p':
			case 'P':
				priv->tcp_flags |= TCP_PSH;
				break;

			case 'a':
			case 'A':
				priv->tcp_flags |= TCP_ACK;
				break;

			case 'u':
			case 'U':
				priv->tcp_flags |= TCP_URG;
				break;

			case '1':
			case 'E':
			case 'e':
				priv->tcp_flags |= TCP_ECE;
				break;

			case '2':
			case 'C':
			case 'c':
				priv->tcp_flags |= TCP_CWR;
				break;

			case '!':
				priv->mode = MODE_NOT;
				break;

			case '*':
				priv->mode = MODE_ANY;
				break;

			case '+':
				priv->mode = MODE_ALL;
				break;

			case ' ':
			case '\t':
			case ',':
				break;
			default:
				my_free(priv);
				return NULL;
		}
	}
	
	return priv;
}

int ack_match(void *priv, p_packet p, int negate)
{
	uint ack;
	p_range_priv x=(p_range_priv)priv;
	
	if ( !CHECK_TCP(p) ) return 0;

	ack=ntohl(TRANSPORT_LAYER(p).h.tcp->ack);
	
	if ( (ack >= x->min) && (ack <= x->max) ) {
		MATCH_RET_POSITIVE;
	}else{
		MATCH_RET_NEGATIVE;
	}
}

int win_match(void *priv, p_packet p, int negate)
{
	ushort win;
	p_range_priv x=(p_range_priv)priv;
	
	if ( !CHECK_TCP(p) ) return 0;

	win=ntohs(TRANSPORT_LAYER(p).h.tcp->win);
	
	if ( (win >= x->min) && (win <= x->max) ) {
		MATCH_RET_POSITIVE;
	}else{
		MATCH_RET_NEGATIVE;
	}
}

int urp_match(void *priv, p_packet p, int negate)
{
	ushort urp;
	p_range_priv x=(p_range_priv)priv;
	
	if ( !CHECK_TCP(p) ) return 0;

	urp=ntohs(TRANSPORT_LAYER(p).h.tcp->urp);
	
	if ( (urp >= x->min) && (urp <= x->max) ) {
		MATCH_RET_POSITIVE;
	}else{
		MATCH_RET_NEGATIVE;
	}
}


int seq_match(void *priv, p_packet p, int negate)
{
	uint seq;
	p_range_priv x=(p_range_priv)priv;
	
	if ( !CHECK_TCP(p) ) return 0;

	seq=ntohl(TRANSPORT_LAYER(p).h.tcp->seq);
	
	if ( (seq >= x->min) && (seq <= x->max) ) {
		MATCH_RET_POSITIVE;
	}else{
		MATCH_RET_NEGATIVE;
	}
}


int flags_match(void *priv, p_packet p, int negate)
{
	p_tcpflag_priv x=priv;
	unsigned char flags;
	
	if ( !CHECK_TCP(p) ) return 0;

	flags=TRANSPORT_LAYER(p).h.tcp->flags.flags;
	
	switch ( x->mode ) {
		case MODE_NORMAL:
			if ( flags == x->tcp_flags ) {
				MATCH_RET_POSITIVE;
			}else{
				MATCH_RET_NEGATIVE;
			}
		case MODE_ALL:
			if ( (flags & x->tcp_flags) == x->tcp_flags ) {
				MATCH_RET_POSITIVE;
			}else{
				MATCH_RET_NEGATIVE;
			}
		case MODE_NOT:
			if ( (flags & x->tcp_flags) == 0) {
				MATCH_RET_POSITIVE;
			}else{
				MATCH_RET_NEGATIVE;
			}
		case MODE_ANY:
			if ( (flags & x->tcp_flags) != 0) {
				MATCH_RET_POSITIVE;
			}else{
				MATCH_RET_NEGATIVE;
			}
	}
	
	print_raw(PRINT_WARNING "Wrong TCP mode, should never happen" CRLF);
	return 0;
}
	
uint _PLUGIN_INIT (p_plugin_in in, p_plugin_out out)
{
	PLUGIN_CHECK_PARAMS(in);

	PLUGIN_IMPORT_STD;

	add_matcher=in->add_matcher;

	PLUGIN_NAME("match/tcp");
	PLUGIN_DESC("Some generic TCP based matchers");
	PLUGIN_VERS(1,0);
	
	add_matcher("FLAGS",
		flags_validate,
		in->templates.t_cleanup_ptr,
		flags_match);
	
	add_matcher("SEQ",
		in->templates.t_validate_range,
		in->templates.t_cleanup_ptr,
		seq_match);

	add_matcher("ACK",
		in->templates.t_validate_range,
		in->templates.t_cleanup_ptr,
		ack_match);
	
	add_matcher("WIN",
		in->templates.t_validate_range,
		in->templates.t_cleanup_ptr,
		win_match);
	
	add_matcher("URP",
		in->templates.t_validate_range,
		in->templates.t_cleanup_ptr,
		urp_match);

	
	
	return PLUGIN_ERR_OK;
}

/* This is where we would cleanup our shit */
uint _PLUGIN_UNLOAD (void)
{
	return PLUGIN_ERR_OK;
}
