/*
* This file is part of Firestorm NIDS
* Copyright (c) 2001 Gianni Tedesco
*
*    Firestorm is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    Firestorm is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with Firestorm; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <capdev.h>
#include <cleanup.h>
#include <s_globals.h>
#include <s_util.h>

/* Lock protects both lists */
p_captype captype=NULL;
p_capdev capdevs=NULL;
mlock_t captlock;


int capdev_init (void)
{
	MUTEX_CREATE(captlock, "Captlock");
	cleanup_add_handler(cleanup_lock, (void *)&captlock);
	return 1;
}


int capdev_add(char *type, char *interface, char *filter, proc_thread proc)
{
	p_captype t=NULL;
	p_capdev  d=globals.malloc(sizeof(capdev_t));
	
	if ( !d ) return 0;

	if ( (t=capdev_find_type(type)) == NULL ) {
		print_out(PRINT_ERROR "Cannot find captype \"%s\"" CRLF, type);
		globals.free(d);
		return 0;
	}

	d->capture=t;
	d->state=CAP_STATE_NULL;
	d->filter=filter;
	d->priv=NULL; /* not set yet */
	d->thread_proc=proc;
	
	if ( strcasecmp(interface, "any") ) {
		d->interface=globals.fs_strdup(interface);
	}else{
		d->interface= NULL;
	}
	
	CRITICAL_START(captlock);
	capdevs=list_append(d, capdevs);
	CRITICAL_STOP(captlock);
	
	return 1;
}

void capdev_init_all(void)
{
	p_capdev dev;

	for (dev=capdevs; dev; dev=(p_capdev)dev->l.next) {
		if ( dev->state  != CAP_STATE_NULL ) continue;
		dev->priv = dev->capture->init(dev->interface, dev->filter);

		if ( dev->priv ) {
			dev->state = CAP_STATE_INIT;
		}else{
			dev->state = CAP_STATE_NULL;
		}
	}	
}

void capdev_start_all(void)
{
	p_capdev dev;

	for (dev=capdevs; dev; dev=(p_capdev)dev->l.next) {
		if ( dev->state != CAP_STATE_INIT ) {
			print_out("%s capture engine: FAILED [%s]" CRLF,
				dev->capture->name, dev->interface);
			continue;
		}
		
		pthread_create (&dev->thread,NULL,dev->thread_proc,dev);

		if ( dev->priv ) {
			print_out("%s capture engine: started [%s]" CRLF,
				dev->capture->name, dev->interface);
			atomic_inc_counter(&globals.keepalive);
		}else{
			print_out("%s capture engine: failed [%s]" CRLF,
				dev->capture->name, dev->interface);
		}
	}

	cleanup_add_handler(capdev_end_all, NULL);
}

void capdev_end_all(int code, void *priv)
{
	p_captype tmp=NULL;
	p_captype foo=NULL;

	p_capdev dev=NULL;
	p_capdev bar=NULL;

	print_raw("Stopping all capture devices" CRLF);
	
	CRITICAL_START(captlock);
	
	/* Remove all captypes */
	for (tmp=captype; tmp;) {
		foo=tmp;
		tmp=(p_captype)tmp->l.next;
		globals.free(foo);
		foo=NULL;
	}

	/* Remove all capdevs */
	for (dev=capdevs; dev; dev=(p_capdev)dev->l.next) {
		if ( dev->state == CAP_STATE_NULL ) continue;

		if ( dev->thread ) pthread_cancel(dev->thread);
		dev->capture->end(dev->priv);
		dev->state = CAP_STATE_NULL;
	}
	for ( dev=capdevs; dev;) {
		bar=dev;
		dev=(p_capdev)dev->l.next;
		if (bar->interface)globals.free(bar->interface);
		if (bar->filter)globals.free(bar->filter);
		globals.free(bar);
		bar=NULL;
	}
	
	CRITICAL_STOP(captlock);
}

p_captype capdev_find_type(char *name)
{
	p_captype tmp=NULL;
	if ( name == NULL ) return NULL;
	for (tmp=captype; tmp; tmp=(p_captype)tmp->l.next) {
		if ( strcmp(name, tmp->name) == 0 ) {
			return tmp;
		}
	}
	return NULL;
}


int capdev_add_type( char *name, proc_capdev_init i, 
       proc_capdev_end e, proc_capdev_poll p)
{
	p_captype type=NULL;
	
	if ( !name || !i || !e || !p ) return 0;
	
	if ( !(type=globals.malloc(sizeof(captype_t))) ) return 0;

	type->name=name;
	type->init=i;
	type->end=e;
	type->poll=p;
	
	/* Add the captype */
	CRITICAL_START(captlock);
	globals.num_captypes++;
	captype=list_append(type, captype);
	CRITICAL_STOP(captlock);

	return 1;
}
